import xarray as xr
import numpy as np
import cartopy.crs as ccrs


def read_ahps(ahps_filename):

    """
    Read and process the AHPS (Advanced Hydrologic Prediction Service) dataset from a given file. The AHPS dataset is
    transformed to Plate Carree projection and invalid observation values are replaced with NaN.

            Parameters
            ----------
            ahps_filename : str
                Filename for the AHPS dataset.

            Returns
            -------
            lon_platecarre : numpy.ndarray
                Longitude values in Plate Carree projection.

            lat_platecarre : numpy.ndarray
                Latitude values in Plate Carree projection.

            obs : numpy.ndarray
                Observation values from the AHPS dataset, with invalid values replaced with NaN and converted to millimeters.

        Example
        --------
        >>> read_ahps("my_ahps_dataset.nc")
        (array([[-180., -178., ...,  178.,  180.],
                [-180., -178., ...,  178.,  180.],
                ...,
                [-180., -178., ...,  178.,  180.],
                [-180., -178., ...,  178.,  180.]]),
         array([[-90., -90., ..., -90., -90.],
                [-88., -88., ..., -88., -88.],
                ...,
                [ 88.,  88., ...,  88.,  88.],
                [ 90.,  90., ...,  90.,  90.]]),
         array([[nan, nan, ..., nan, nan],
                [nan, nan, ..., nan, nan],
                ...,
                [nan, nan, ..., nan, nan],
                [nan, nan, ..., nan, nan]]))

    """

    ahps = xr.open_dataset(ahps_filename)
    crs = ahps["crs"]
    lon = ahps.variables["x"][:]
    lat = ahps.variables["y"][:]

    polar_proj = ccrs.NorthPolarStereo(
        central_longitude=crs.attrs["straight_vertical_longitude_from_pole"],
        true_scale_latitude=crs.attrs["standard_parallel"],
        globe=None,
    )
    plate_proj = ccrs.PlateCarree(central_longitude=0.0, globe=None)
    x_mesh, y_mesh = np.meshgrid(lon, lat)
    ahps_latlon = plate_proj.transform_points(polar_proj, x_mesh, y_mesh)
    lon_platecarre = ahps_latlon[:, :, 0]
    lat_platecarre = ahps_latlon[:, :, 1]
    obs = ahps.variables["observation"].values * 25.4
    obs[obs <= 0] = np.nan

    return lon_platecarre, lat_platecarre, obs



"""
A collection of different metrics that work on finite-dimensional collections of numbers
"""

from numpy import array
from math import sqrt

def l1(x_in, y_in):
    x = array(x_in).flatten()
    y = array(y_in).flatten()
    return sum(abs(x-y))

def l2(x_in, y_in):
    x = array(x_in).flatten()
    y = array(y_in).flatten()
    return sqrt(sum((x-y)**2))

def sup(x_in, y_in):
    x = array(x_in).flatten()
    y = array(y_in).flatten()
    return max(abs(x-y))

def hamming(x_in, y_in):
    x = array(x_in).flatten()
    y = array(y_in).flatten()
    # for some reason, `-` is evaluated as component-wise xor
    difference = (abs(x) >=1 ) - (abs(y) >=1 )
    print(difference)
    return sum( difference * 1)



import matplotlib.pyplot as plt
import cartopy.crs as ccrs

import cartopy.feature as cfeature
import cartopy.crs as ccrs
from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER


# def plot_coast(axes):
def plot_coast(axes: cartopy.mpl.geoaxes.GeoAxes) -> None:
    """
    Plot natural features and gridlines on a map using Cartopy.

    Parameters
    ----------
    axes : cartopy.mpl.geoaxes.GeoAxes
        The axes object to plot on.

    Returns
    -------
    None

    Example
    --------
    >>> import matplotlib.pyplot as plt
    >>> import cartopy.crs as ccrs

    >>> #Set up the figure and axes
    >>> fig, ax = plt.subplots(1, 1, figsize=(10, 10))

    >>> #Set the projection for the axes
    >>> ax.projection = ccrs.PlateCarree()

    >>> #Call the plot_coast function
    >>> plot_coast(ax)

    >>> #Show the plot
    >>> plt.show()

    """
    countries = cfeature.NaturalEarthFeature(
        scale="50m", category="cultural", name="admin_0_countries", facecolor="none"
    )
    states = cfeature.NaturalEarthFeature(
        scale="50m",
        category="cultural",
        name="admin_1_states_provinces_lines",
        facecolor="none",
    )
    axes.add_feature(countries, edgecolor="k", linewidth=0.5)
    axes.add_feature(states, edgecolor="k", linewidth=0.5)
    gl = axes.gridlines(
        crs=ccrs.PlateCarree(),
        draw_labels=True,
        linewidth=2,
        color="gray",
        alpha=0,
        linestyle="--",
    )
    gl.top_labels = False
    gl.bottom_labels = True
    gl.left_labels = True
    gl.right_labels = False
    gl.xlines = True
    gl.xformatter = LONGITUDE_FORMATTER
    gl.yformatter = LATITUDE_FORMATTER



from setuptools import setup

setup(
    # Needed to silence warnings
    name='selfutils',
    url='https://github.com/ankurk017/pyhurr',
    author='Ankur Kumar',
    author_email='ankurk017@gmail.com',
    # Needed to actually package something
    packages=['self_utils'],
    # Needed for dependencies
    install_requires=["cartopy", "matplotlib", "numpy", "xarray"],
    # *strongly* suggested for sharing
    version='0.1.1',
    license='MIT',
    description='An example of a python package from pre-existing code',
    # We will also need a readme eventually (there will be a warning)
    long_description=open('README.rst').read(),
)



"""
A collection of different norms that work on finite-dimensional collections of numbers
"""

from numpy import array
from math import sqrt

def l1(x_in):
    x = array(x_in).flatten()
    return sum(abs(x))

def l2(x_in):
    x = array(x_in).flatten()
    return sqrt(sum(x**2))

def sup(x_in):
    x = array(x_in).flatten()
    return max(abs(x))

def hamming(x_in):
    x = array(x_in).flatten()
    return sum( ( abs(x) >= 1) * 1 )
